/**
 * @file main.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2014
 * License GPLv3+: GNU GPL Version 3
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * This file starts the daemon and handles program options.
**/
#include "config.h"
#include "lindmxd.h"
#include "lindmxd_config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <syslog.h>

const char *program_name;


/**
 * print_usage()
 * Prints the program usage
 * @param stream     IOSTREAM to print to
 * @param exit_code  The code to give to exit() after printing
**/
void print_usage(FILE* stream, int exit_code)
{
	fprintf(stream, "Usage:  %s [options]\n", program_name);
	fprintf(stream,
			"  -h  --help            Display this usage information\n"
			"  -v  --version         Display program version\n"
			"  -p  --port n          Set the port to n\n"
			"  -c  --channels n      Set maximum number of channels to n\n"
			"  -i  --interface type  Select interface type\n"
			"  -l  --list            Prints a list of available interface types\n"
			"  -d  --debug           Disable fork and give debug messages\n"
			"  -z  --zero            Set all channels to zero on start-up\n");
	/* additional information */
	fprintf(stream, "\nThis  service is logged by syslog. Watch your logs to fix problems.\n");
	exit(exit_code);
}



/**
 * print_interface_list()
 * Prints a list of available interfaces
 * @param stream     IOSTREAM to print to
 * @param exit_code  The code to give to exit() after printing
**/
void print_interface_list(FILE* stream, int exit_code)
{
	fprintf(stream, "Available interfaces:\n");
	fprintf(stream,
			"   Name:                  |   Description: \n"
			"   -----------------------+----------------------------------------\n"
			"   simulate               |   Simulates an interface\n"
			"   dmx4all_micro_usb      |   DMX4ALL Micro USB Interface\n"
			"   dmx4all_tcp            |   DMX4ALL Stage Profi LAN via tcp\n"
			"   dmx4all_udp            |   DMX4ALL Stage Profi LAN via udp\n"
			"   -----------------------+----------------------------------------\n");
	exit(exit_code);
}



/**
 * print_version()
 * Prints the program version
**/
void print_version()
{
	printf("%s: Version %d.%d.%d-%d\n", program_name, LINDMX_MAJOR_VERSION,
	                                                  LINDMX_MINOR_VERSION,
	                                                  LINDMX_PATCH_VERSION,
	                                                  LINDMX_BUILD_NUMBER);
	printf("Copyright © 2012-2014 Florian Schmitt\n"
	       "License GPLv3+: GNU GPL Version 3 or higher <http://gnu.org/licenses/gpl.html>\n"
	       "This program comes with ABSOLUTELY NO WARRANTY.\n"
	       "This is free software, and you are welcome to \n"
	       "redistribute it under certain conditions.\n\n"
	       "This software is written by Florian Schmitt <florian.schmitt@qikgrp.com>\n");
	exit(EXIT_SUCCESS);
}



/**
 * main()
 * Main procedure
**/
int main(int argc, char* argv[])
{
	int retval;
	lindmxd_config_t conf;
	int  next_option;
	char interface[100];

	/* options */
	const char* const short_options = "hvp:c:si:ldz";
	const struct option long_options[] = {
		{ "help",     0, NULL, 'h' },
		{ "version",  0, NULL, 'v' },
		{ "port",     1, NULL, 'p' },
		{ "channels", 1, NULL, 'c' },
		{ "simulate", 0, NULL, 's' },
		{ "interface",1, NULL, 'i' },
		{ "list",     0, NULL, 'l' },
		{ "debug",    0, NULL, 'd' },
		{ "zero",     0, NULL, 'z' },
		{ NULL,       0, NULL,  0  }
	};

	/* default values */
	lindmxd_config_init_defaults(&conf);
	program_name      = argv[0];
	conf.program_name = argv[0];

	/* Parse config file */
	printf("Loading config from %s\n", LINDMXD_CONFIG);
	retval = lindmxd_config_parse(&conf, LINDMXD_CONFIG, 5);
	if( retval != LINDMXD_CONFIG_SUCCESS )
	{
		fprintf(stderr, "ERROR: %s (%d)\n", lindmxd_config_geterr(), retval);
		exit(1);
	}

	/* get the options */
	do {
		next_option = getopt_long(argc, argv, short_options, long_options, NULL);
		switch(next_option)
		{
			case 'h':
				print_usage(stdout, EXIT_SUCCESS);
			case 'p':
				conf.port = atoi(optarg);
				break;
			case 'c':
				conf.channels = atoi(optarg);
				break;
			case 'd':
				conf.debug    = 1;
				conf.log_upto = LOG_DEBUG;
				break;
			case 's':
				printf("Simulation mode is deprecated. See --help for info.\n");
				printf("Use 'simulate' as interface name.\n");
				exit(EXIT_FAILURE);
				break;
			case 'i':
				strncpy(interface, optarg, 100);
				conf.interface = interface;
				break;
			case 'l':
				print_interface_list(stdout, EXIT_SUCCESS);
			case 'v':
				print_version();
				break;
			case 'z':
				conf.zero = 0;
				break;
			case '?':
				print_usage(stderr, EXIT_FAILURE);
				break;
			case -1:
				break;
			default:
				abort();
		}
	} while(next_option != -1);

	/* start up daemon */
	if(conf.debug == 1) printf("Starting lindmxd on port %d providing %d DMX channels.\nInterface type: '%s' [DEBUG=ON]\n", conf.port, conf.channels, conf.interface);
	else
	{
		printf("Starting lindmxd on port %d providing %d DMX channels.\nInterface type: '%s'\n", conf.port, conf.channels, conf.interface);
		printf("Being silent now. See syslog for errors or use -d\n");
	}
	lindmxd_start( &conf );

	abort();
 	return 0;
}
